const toArray = (e) => Array.from(e) || [];

const insertAfter = (newElement, targetElement) => {
    var parent = targetElement.parentNode;
    if (parent.lastChild == targetElement) {
        parent.appendChild(newElement);
    } else {
        parent.insertBefore(newElement, targetElement.nextSibling);
    }
}

/**
 * console.cloud.tencent.com
 * cvm
 */
const CVMPlugin = (target) => {
    const pathname = window.location.pathname;
    const btnKeyID = "CloudStudioEntrance";

    const handleTrInfoDOM = (tbody) => {
        const allTr = toArray(tbody.getElementsByTagName("tr"));
        // 获取 root
        const getRootName = (target) => {
            const hasRootTd = toArray(target.childNodes)[5];
            const i = hasRootTd.getElementsByTagName("I")[0];
            const nums = i.className.match(/[0-9]/g);
            const map = {
                "1": "root",
                "6": "ubuntu"
            };

            return Array.isArray(nums) ? (map[nums[0]] || "") : ""
        }

        const isCanNext = (rootName) => rootName === "ubuntu" || rootName === "root";

        allTr.forEach(tr => {
            const lastTd = tr.lastElementChild;

            // 指定需要有 app-cvm-btn 类名的按钮才可插入入口;
            const loginBtn = lastTd.getElementsByClassName("app-cvm-btn")[0];
            if (!loginBtn) return;

            // 如果添加过则不添加
            if (lastTd.querySelector(`#${btnKeyID}`)) return;

            // 仅支持 ubutu 和 centos 
            const rootName = getRootName(tr);
            if (!isCanNext(rootName)) return;

            const newBtnEl = document.createElement("div");
            newBtnEl.id = btnKeyID;
            newBtnEl.className = loginBtn.className;
            newBtnEl.innerText = "云端开发";
            newBtnEl.onclick = () => {
                // 获取 ip
                const getIp = () => {
                    const hasIpTd = toArray(tr.childNodes)[7];
                    const text = hasIpTd.innerText;
                    const toNumber = text.split(".").map(e => parseInt(e));
                    const ip = toNumber.slice(0, 4).join(".");
                    return ip || ""
                }

                window.open(`https://cloudstudio.net/#create-from-cloud=${getIp()}_-_${rootName}`);
            }

            insertAfter(newBtnEl, loginBtn);

        })
    }

    if (pathname.includes("cvm/instance")) {
        const tableBody = target.getElementsByClassName("app-cvm-table__body")[0];
        if (!tableBody) return;

        const tbody = tableBody.getElementsByClassName("tbody")[0];

        const observer = new MutationObserver(function (mutationsList) {
            mutationsList.forEach(el => {
                // 保证是 cvm 主机列表更新
                if (el.type === "childList" && el.target && el.target.tagName === "TBODY") {
                    handleTrInfoDOM(el.target)
                }
            })
        });


        observer.observe(tbody || tableBody, { attributes: true, childList: true, subtree: true });
    }
};

document.addEventListener("readystatechange", (e) => {
    if (document.readyState === "complete") {
        CVMPlugin(e.target);
    }
});